data = {
    "tinyfiledialogs": {
        "type": "library",
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["zlib"],
                "changelog": "https://sourceforge.net/p/tinyfiledialogs/activity/?page=0&limit=100#65c79b2c7e19948b1b33b56a",
                "prereqs": [
                    "{triplet}-gcc",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://git.code.sf.net/p/tinyfiledialogs/code",
                    "files": ["README.txt"],
                    "hint": r'tiny file dialogs \( cross-platform C C\+\+ \) v\d\.\d+(?:\.\d)?',
                },
                "installer": {
                    "type": "custom",
                    "name": "tinyfiledialogs",
                },
            },
        ],
    },
    "unzip": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://infozip.sourceforge.net/UnZip.html#Release",
                "checker": {
                    "type": "pacman",
                    "package": "unzip",
                },
                "installer": {
                    "type": "pacman",
                    "package": "unzip",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://infozip.sourceforge.net/UnZip.html#Release",
                "checker": {
                    "type": "apt-cache",
                    "package": "unzip",
                },
                "installer": {
                    "type": "apt",
                    "package": "unzip",
                },
            },
        ],
    },
    "valgrind": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://valgrind.org/docs/manual/dist.news.html",
                "checker": {
                    "type": "pacman",
                    "package": "valgrind",
                },
                "installer": {
                    "type": "pacman",
                    "package": ["valgrind", "debuginfod"],
                },
            },
        ],
    },
    "wayland-libs": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://wayland.freedesktop.org/releases.html",
                "prereqs": [
                    "{triplet}-g++",
                    "{triplet}-gcc",
                    "{triplet}-pkg-config",
                    "libffi",
                    "meson",
                    "ninja",
                    "wayland-scanner",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/wayland/wayland.git",
                    "hint": r'1\.\d\d.\d+',
                },
                "installer": {
                    "type": "custom",
                    "name": "wayland",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://wayland.freedesktop.org/releases.html",
                "checker": {
                    "type": "pacman",
                    "package": "wayland",
                },
                "installer": {
                    "type": "pacman",
                    "package": "wayland",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "https://wayland.freedesktop.org/releases.html",
                "checker": {
                    "type": "apt-cache",
                    "package": "libwayland-egl-backend-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "wayland-scanner": {
        "platform": "host",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://wayland.freedesktop.org/releases.html",
                "prereqs": [
                    "expat",
                    "gcc",
                    "meson",
                    "ninja",
                    "pkg-config",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/wayland/wayland.git",
                    "hint": r'1\.\d\d.\d+',
                },
                "installer": {
                    "type": "custom",
                    "name": "wayland-scanner",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://wayland.freedesktop.org/releases.html",
                "checker": {
                    "type": "pacman",
                    "package": "wayland",
                },
                "installer": {
                    "type": "pacman",
                    "package": "wayland",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://wayland.freedesktop.org/releases.html",
                "checker": {
                    "type": "apt-cache",
                    "package": "libwayland-bin",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "wget": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=wget",
                "checker": {
                    "type": "pacman",
                    "package": "wget",
                },
                "installer": {
                    "type": "pacman",
                    "package": "wget",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=wget",
                "checker": {
                    "type": "yum",
                    "package": "wget",
                },
                "installer": {
                    "type": "yum",
                    "package": "wget",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=wget",
                "checker": {
                    "type": "apt-cache",
                    "package": "wget",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "x86_64-linux-gnu-binutils": {
        "platform": "host",
        "projects": [
            {
                "arch": ["x86_64"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/binutils/binutils/NEWS",
                "checker": {
                    "type": "yum",
                    "package": "binutils",
                },
                "installer": {
                    "type": "cmd",
                    "cmds": [
                        ["mkdir", "-p", "/usr/local/bin"],
                        [
                            "ln", "-s", "/usr/bin/ar",
                            "/usr/local/bin/x86_64-linux-gnu-ar"
                        ],
                        [
                            "ln", "-s", "/usr/bin/strip",
                            "/usr/local/bin/x86_64-linux-gnu-strip"
                        ],
                    ]
                },
            },
            {
                "arch": ["x86_64"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/binutils/binutils/NEWS",
                "checker": {
                    "type": "apt-cache",
                    "package": "binutils-x86-64-linux-gnu",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "x86_64-linux-gnu-g++": {
        "platform": "host",
        "projects": [
            {
                "arch": ["x86_64"],
                "os": ["archlinux"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "prereqs": ["g++"],
                "checker": {
                    "type": "pacman",
                    "package": "gcc",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
            {
                "arch": ["x86_64"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "prereqs": ["g++"],
                "checker": {
                    "type": "yum",
                    "package": "g++",
                },
                "installer": {
                    "type": "cmd",
                    "cmds": [
                        ["mkdir", "-p", "/usr/local/bin"],
                        [
                            "ln", "-s", "/usr/bin/g++",
                            "/usr/local/bin/x86_64-linux-gnu-g++"
                        ],
                    ]
                },
            },
            {
                "arch": ["x86_64"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "prereqs": [
                    "gcc",
                ],
                "checker": {
                    "type": "apt-cache",
                    "package": "gcc-12-monolithic",
                },
                "installer": {
                    "type": "cmd",
                    "cmds": [
                        [
                            "update-alternatives", "--set",
                            "x86_64-linux-gnu-g++", "/usr/bin/g++-12", "1"
                        ],
                    ],
                },
            },
        ],
    },
    "x86_64-linux-gnu-gcc": {
        "platform": "host",
        "projects": [
            {
                "arch": ["x86_64"],
                "os": ["archlinux"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "prereqs": ["gcc"],
                "checker": {
                    "type": "pacman",
                    "package": "gcc",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
            {
                "arch": ["x86_64"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "prereqs": ["gcc"],
                "checker": {
                    "type": "yum",
                    "package": "gcc",
                },
                "installer": {
                    "type": "cmd",
                    "cmds": [
                        ["mkdir", "-p", "/usr/local/bin"],
                        [
                            "ln", "-s", "/usr/bin/gcc",
                            "/usr/local/bin/x86_64-linux-gnu-gcc"
                        ],
                        [
                            "ln", "-s", "/usr/bin/gcc-ar",
                            "/usr/local/bin/x86_64-linux-gnu-gcc-ar"
                        ],
                        [
                            "ln", "-s", "/usr/bin/gcc-nm",
                            "/usr/local/bin/x86_64-linux-gnu-gcc-nm"
                        ],
                        [
                            "ln", "-s", "/usr/bin/gcc-ranlib",
                            "/usr/local/bin/x86_64-linux-gnu-gcc-ranlib"
                        ],
                        [
                            "ln", "-s", "/usr/bin/gcov",
                            "/usr/local/bin/x86_64-linux-gnu-gcov"
                        ],
                    ]
                },
            },
            {
                "arch": ["x86_64"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "checker": {
                    "type": "apt-cache",
                    "package": "gcc-12-monolithic",
                },
                "installer": {
                    "type": "cmd",
                    "cmds": [
                        [
                            "update-alternatives", "--set",
                            "x86_64-linux-gnu-cpp", "/usr/bin/cpp-12", "1"
                        ],
                        [
                            "update-alternatives", "--set",
                            "x86_64-linux-gnu-gcc", "/usr/bin/gcc-12", "1"
                        ],
                        [
                            "update-alternatives", "--set",
                            "x86_64-linux-gnu-gcc-ar", "/usr/bin/gcc-ar-12",
                            "1"
                        ],
                        [
                            "update-alternatives", "--set",
                            "x86_64-linux-gnu-gcc-nm", "/usr/bin/gcc-nm-12",
                            "1"
                        ],
                        [
                            "update-alternatives", "--set",
                            "x86_64-linux-gnu-gcc-ranlib",
                            "/usr/bin/gcc-ranlib-12", "1"
                        ],
                        [
                            "update-alternatives", "--set",
                            "x86_64-linux-gnu-gcov", "/usr/bin/gcov-12", "1"
                        ]
                    ],
                },
            },
        ],
    },
    "x86_64-linux-gnu-pkg-config": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "changelog": "https://gitlab.freedesktop.org/pkg-config/pkg-config/-/blob/master/NEWS?ref_type=heads",
                "prereqs": [
                    "pkg-config",
                ],
                "checker": {
                    "type": "pacman",
                    "package": "pkg-config",
                },
                "installer": {
                    "type": "cmd",
                    "cmds": [
                        ["mkdir", "-p", "/usr/local/bin/"],
                        ["sh", "-c", "echo "
                            "'#!/bin/sh\n"
                            "export PKG_CONFIG_DIR=\n"
                            "export PKG_CONFIG_PATH="
                                "/usr/local/x86_64-linux-gnu/lib/pkgconfig:"
                                "/usr/local/x86_64-linux-gnu/share/pkgconfig"
                                "\n"
                            "export PKG_CONFIG_LIBDIR="
                                "/usr/local/x86_64-linux-gnu/lib/pkgconfig:"
                                "/usr/local/x86_64-linux-gnu/share/pkgconfig"
                                "\n"
                            "export PKG_CONFIG_SYSROOT_DIR=/\n"
                            "exec pkg-config \"$@\"' "
                            "> /usr/local/bin/x86_64-linux-gnu-pkg-config",
                        ],
                        [
                            "chmod", "+x",
                            "/usr/local/bin/x86_64-linux-gnu-pkg-config",
                        ],
                    ],
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["gnu"],
                "changelog": "https://gitlab.freedesktop.org/pkg-config/pkg-config/-/blob/master/NEWS?ref_type=heads",
                "prereqs": [
                    "pkg-config",
                ],
                "checker": {
                    "type": "yum",
                    "package": "pkgconfig",
                },
                "installer": {
                    "type": "cmd",
                    "cmds": [
                        ["mkdir", "-p", "/usr/local/bin/"],
                        ["sh", "-c", "echo "
                            "'#!/bin/sh\n"
                            "export PKG_CONFIG_DIR=\n"
                            "export PKG_CONFIG_PATH="
                                "/usr/local/x86_64-linux-gnu/lib/pkgconfig:"
                                "/usr/local/x86_64-linux-gnu/share/pkgconfig"
                                "\n"
                            "export PKG_CONFIG_LIBDIR="
                                "/usr/local/x86_64-linux-gnu/lib/pkgconfig:"
                                "/usr/local/x86_64-linux-gnu/share/pkgconfig"
                                "\n"
                            "export PKG_CONFIG_SYSROOT_DIR=/\n"
                            "exec pkg-config \"$@\"' "
                            "> /usr/local/bin/x86_64-linux-gnu-pkg-config",
                        ],
                        [
                            "chmod", "+x",
                            "/usr/local/bin/x86_64-linux-gnu-pkg-config",
                        ],
                    ],
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "prereqs": [
                    "pkg-config",
                ],
                "checker": {
                    "type": "apt-cache",
                    "package": "pkg-config",
                },
                "installer": {
                    "cmds": [
                        ["mkdir", "-p", "/usr/local/bin/"],
                        ["sh", "-c", "echo "
                            "'#!/bin/sh\n"
                            "export PKG_CONFIG_DIR=\n"
                            "export PKG_CONFIG_PATH="
                                "/usr/local/x86_64-linux-gnu/lib/pkgconfig:"
                                "/usr/local/x86_64-linux-gnu/share/pkgconfig"
                                "\n"
                            "export PKG_CONFIG_LIBDIR="
                                "/usr/local/x86_64-linux-gnu/lib/pkgconfig:"
                                "/usr/local/x86_64-linux-gnu/share/pkgconfig"
                                "\n"
                            "export PKG_CONFIG_SYSROOT_DIR=/\n"
                            "exec pkg-config \"$@\"' "
                            "> /usr/local/bin/x86_64-linux-gnu-pkg-config",
                        ],
                        [
                            "chmod", "+x",
                            "/usr/local/bin/x86_64-linux-gnu-pkg-config",
                        ],
                    ],
                },
            },
        ],
    },
    "x86_64-w64-mingw32-g++": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://www.mingw-w64.org/changelog/",
                "prereqs": ["x86_64-w64-mingw32-gcc"],
                "checker": {
                    "type": "pacman",
                    "package": "mingw-w64-gcc",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "x86_64-w64-mingw32-gcc": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://www.mingw-w64.org/changelog/",
                "checker": {
                    "type": "pacman",
                    "package": "mingw-w64-gcc",
                },
                "installer": {
                    "type": "pacman",
                    "package": "mingw-w64-gcc",
                },
            },
        ],
    },
    "x86_64-w64-mingw32-pkg-config": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "changelog": "https://gitlab.freedesktop.org/pkg-config/pkg-config/-/blob/master/NEWS?ref_type=heads",
                "prereqs": [
                    "pkg-config",
                ],
                "checker": {
                    "type": "pacman",
                    "package": "pkgconf",
                },
                "installer": {
                    "cmds": [
                        ["mkdir", "-p", "/usr/local/bin/"],
                        ["sh", "-c", "echo "
                            "'#!/bin/sh\n"
                            "export PKG_CONFIG_DIR=\n"
                            "export PKG_CONFIG_PATH="
                                "/usr/local/x86_64-w64-mingw32/lib/pkgconfig:"
                                "/usr/local/x86_64-w64-mingw32/share/pkgconfig"
                                "\n"
                            "export PKG_CONFIG_LIBDIR="
                                "/usr/local/x86_64-w64-mingw32/lib/pkgconfig:"
                                "/usr/local/x86_64-w64-mingw32/share/pkgconfig"
                                "\n"
                            "export PKG_CONFIG_SYSROOT_DIR=/\n"
                            "exec pkg-config \"$@\"' "
                            ">> /usr/local/bin/x86_64-w64-mingw32-pkg-config",
                        ],
                        [
                            "chmod", "+x",
                            "/usr/local/bin/x86_64-w64-mingw32-pkg-config",
                        ],
                    ],
                },
            },
        ],
    },
    "xau": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libxau/-/tags",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-pkg-config",
                    "autoconf",
                    "libtool",
                    "make",
                    "xorg-macros",
                    "xorgproto",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/xorg/lib/libxau.git",
                    "hint": r'libXau-1\.0\.\d\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "xau",
                    "fallback": {
                        "type": "custom",
                        "name": "xau-github",
                    },
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libxau/-/tags",
                "checker": {
                    "type": "pacman",
                    "package": "libxau",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libxau",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libxau/-/tags",
                "checker": {
                    "type": "apt-cache",
                    "package": "libxau-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "xcb": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["x11"],
                "changelog": "https://xcb.freedesktop.org/",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-pkg-config",
                    "autoconf",
                    "make",
                    "pthread-stubs",
                    "wget",
                    "xcb-proto",
                    "xau",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/xorg/lib/libxcb.git",
                    "hint": r'libxcb-1\.\d\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "xcb",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["x11"],
                "changelog": "https://xcb.freedesktop.org/",
                "checker": {
                    "type": "pacman",
                    "package": "libxcb",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libxcb",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["x11"],
                "changelog": "https://xcb.freedesktop.org/",
                "checker": {
                    "type": "apt-cache",
                    "package": "libx11-xcb-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "xcb-proto": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["x11"],
                "changelog": "https://xcb.freedesktop.org/",
                "prereqs": [
                    "autoconf",
                    "automake",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/xorg/proto/xcbproto.git",
                    "hint": r'xcb-proto-1\.\d\d\.\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "xcb-proto",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["x11"],
                "changelog": "https://xcb.freedesktop.org/",
                "checker": {
                    "type": "pacman",
                    "package": "xcb-proto",
                },
                "installer": {
                    "type": "pacman",
                    "package": "xcb-proto",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["x11"],
                "changelog": "https://xcb.freedesktop.org/",
                "checker": {
                    "type": "apt-cache",
                    "package": "xcb-proto",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
}



from libvgazer.command    import RunCommand
from libvgazer.exceptions import CommandError
from libvgazer.exceptions import InstallError

aptUpdateCalled = False

def InstallPackageDebian(software, package, hostPlatform, verbose):
    try:
        RunCommand(
         [
          "apt-get", "-t",
          "{osver}-backports".format(osver=hostPlatform.GetOsVersion()),
          "install", "-y", "--no-install-recommends", "--no-install-suggests",
          package
         ],
         verbose)
    except CommandError as e:
        if e.errorcode == 100:
            InstallPackageOther(software, package, hostPlatform, verbose)
        else:
            print("VGAZER: Unable to install", software)
            raise InstallError(
             "{software} not installed".format(software=software))

def InstallPackageOther(software, package, hostPlatform, verbose):
    try:
        RunCommand(
         ["apt-get", "install", "-y", "--no-install-recommends",
          "--no-install-suggests", package],
         verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software}  not installed".format(software=software))

def InstallPackage(software, package, hostPlatform, verbose):
    if hostPlatform.GetOs() == "debian":
        InstallPackageDebian(software, package, hostPlatform, verbose)
    else:
        InstallPackageOther(software, package, hostPlatform, verbose)

def InstallApt(software, packages, postInstallCommands, hostPlatform, verbose):
    global aptUpdateCalled

    if not aptUpdateCalled:
        RunCommand(["apt-get", "update"], verbose)
        aptUpdateCalled = True

    if isinstance(packages, list):
        for package in packages:
            InstallPackage(software, package, hostPlatform, verbose)
    else:
        InstallPackage(software, packages, hostPlatform, verbose)

    if postInstallCommands is not None:
        for command in postInstallCommands:
            RunCommand(command, verbose)

    print("VGAZER:", software, "installed")



import importlib

from libvgazer.exceptions import MissingInstaller

class InstallCustom:
    def __init__(self, customInstallers={}):
        self.customInstallers = customInstallers

    def AddData(self, customInstallers):
        self.customInstallers = {**self.customInstallers, **customInstallers}

    def Install(self, software, installerName, platform, platformData,
     verbose):
        try:
            installer = importlib.import_module(
             'libvgazer.install.custom_installer.' + installerName)
        except ImportError:
            if len(self.customInstallers) == 0:
                raise MissingInstaller(
                 "Missing custom installer for project: " + software)
            for customInstaller in self.customInstallers:
                if customInstaller["name"] == installerName:
                    installer = customInstaller["installer"]

        return installer.Install(software, platform, platformData, verbose)



import re

from libvgazer.command import GetCommandOutputUtf8

def CheckYum(package, repo, arch):
    cmd = ["yum", "list", package]
    if repo is not None:
        cmd.append("--enablerepo={repo}".format(repo=repo))
    output = GetCommandOutputUtf8(cmd)
    lines = output.splitlines()
    maxVersion = ""

    for line in lines:
        if package in line:
            columns = re.split(' +', line)
            version = columns[1]
            pkgArch = columns[0].split(".")[-1]
            if pkgArch in [arch, "noarch"] and (version > maxVersion):
                maxVersion = version

    return maxVersion



import os
import re

from libvgazer.command     import GetCommandOutputUtf8
from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.store.temp  import StoreTemp
from libvgazer.working_dir import WorkingDir

def GetLastTag(url, hint=None, files=None):
    if files is not None:
        return None

    storeTemp = StoreTemp()
    tempPath = storeTemp.GetDirectoryPath()

    try:
        with WorkingDir(tempPath, verbose=False):
            output = GetCommandOutputUtf8([
             "git", "-c", "versionsort.suffix=-", "ls-remote", "--tags", url]
            )

        if (output == ""):
            return None

        lines = output.splitlines()
        tags = []
        for line in lines:
            if line.endswith("^{}"):
                continue
            tag = line.split("/")[-1]
            if hint is None or re.fullmatch(hint, tag) is not None:
                tags.append(tag)

        tags.sort(reverse=True)
        return tags[0]

    except CommandError:
        return None

    return None

def GetLastCommit(url, hint=None, files=None):
    storeTemp = StoreTemp()
    tempPath = storeTemp.GetDirectoryPath()

    try:
        clonedDir = os.path.join(tempPath, "cloned")
        RunCommand(["rm", "-rf", clonedDir], False)
        RunCommand(["mkdir", "-p", clonedDir], False)

        with WorkingDir(clonedDir, verbose=False):
            RunCommand(["git", "clone", "--depth", "1", url, "."], False)

            if files is not None:
                mostRecent = ""
                for file in files:
                    if hint is None:
                        version = GetCommandOutputUtf8([
                         "git", "--no-pager", "log", "-1", "--format=%ai", file]
                        )[0:-6]
                        if version > mostRecent:
                            mostRecent = version
                    else:
                        with open(file) as openedFile:
                            content = openedFile.read()
                            matches = re.findall(hint, content)
                            for match in matches:
                                if match > mostRecent:
                                    mostRecent = match

                return mostRecent if mostRecent != "" else None

            output = GetCommandOutputUtf8([
             "git", "--no-pager", "log", "-1", "--format=%ai"]
            )
            splitted = output.split(" ")
            return "{date} {time}".format(date=splitted[0], time=splitted[1])
    except CommandError:
        return None

    return None

def CheckGit(url, hint=None, files=None):
    version = GetLastTag(url, hint, files)
    if version is None:
        version = GetLastCommit(url, hint, files)

    return version



data = {
    "xlib": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit", "x11", "hpnd-sv", "smlnj", "hpnd", "bsd-1"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libx11#release-187",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-pkg-config",
                    "autoconf",
                    "make",
                    "xcb",
                    "xorgproto",
                    "xtrans",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/xorg/lib/libx11.git",
                    "hint": r'libX11-\d\.\d+(?:\.\d+(?:\.\d+)?)?',
                },
                "installer": {
                    "type": "custom",
                    "name": "xlib",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["mit", "x11", "hpnd-sv", "smlnj", "hpnd", "bsd-1"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libx11#release-187",
                "checker": {
                    "type": "pacman",
                    "package": "libx11",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libx11",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit", "x11", "hpnd-sv", "smlnj", "hpnd", "bsd-1"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libx11#release-187",
                "checker": {
                    "type": "apt-cache",
                    "package": "libx11-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "xmempool": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/XadillaX/xmempool/tags",
                "prereqs": [
                    "{triplet}-gcc",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/XadillaX/xmempool.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "xmempool",
                },
            },
        ],
    },
    "xorg-macros": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://gitlab.freedesktop.org/xorg/util/macros/-/tags",
                "prereqs": [
                    "autoconf",
                    "automake",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/xorg/util/macros.git",
                    "hint": r'util-macros-1\.\d\d\.\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "xorg-macros",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://gitlab.freedesktop.org/xorg/util/macros/-/tags",
                "checker": {
                    "type": "pacman",
                    "package": "xorg-util-macros",
                },
                "installer": {
                    "type": "pacman",
                    "package": "xorg-util-macros",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://gitlab.freedesktop.org/xorg/util/macros/-/tags",
                "checker": {
                    "type": "apt-cache",
                    "package": "xutils-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "xorgproto": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": [
                    "bsd-2", "hpnd", "hpnd-sv", "icu", "mit", "sgi-b-2",
                    "smlnj", "x11"
                ],
                "changelog": "https://gitlab.freedesktop.org/xorg/proto/xorgproto/-/tags",
                "prereqs": [
                    "{triplet}-gcc",
                    "meson",
                    "ninja",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/xorg/proto/xorgproto.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "xorgproto",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["mit", "smlnj"],
                "changelog": "https://gitlab.freedesktop.org/xorg/proto/xorgproto/-/tags",
                "checker": {
                    "type": "pacman",
                    "package": "xorgproto",
                },
                "installer": {
                    "type": "pacman",
                    "package": "xorgproto",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit", "smlnj"],
                "changelog": "https://gitlab.freedesktop.org/xorg/proto/xorgproto/-/tags",
                "checker": {
                    "type": "apt-cache",
                    "package": "x11proto-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "xtrans": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit", "hpnd", "x11", "hpnd-sv"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libxtrans/-/tags",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-pkg-config",
                    "make",
                    "xorg-macros",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/xorg/lib/libxtrans.git",
                    "hint": r'xtrans-\d\.\d\.\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "xtrans",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["mit", "hpnd", "x11", "hpnd-sv"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libxtrans/-/tags",
                "checker": {
                    "type": "pacman",
                    "package": "xtrans",
                },
                "installer": {
                    "type": "pacman",
                    "package": "xtrans",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit", "hpnd", "x11", "hpnd-sv"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libxtrans/-/tags",
                "checker": {
                    "type": "apt-cache",
                    "package": "xtrans-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "xz": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["pd"],
                "changelog": "https://github.com/tukaani-project/xz/blob/master/NEWS",
                "checker": {
                    "type": "pacman",
                    "package": "xz",
                },
                "installer": {
                    "type": "pacman",
                    "package": "xz",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://github.com/tukaani-project/xz/blob/master/NEWS",
                "checker": {
                    "type": "yum",
                    "package": "xz",
                },
                "installer": {
                    "type": "yum",
                    "package": "xz",
                },
            },
            # Temporary stub
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=coreutils",
                "checker": {
                    "type": "apt-cache",
                    "package": "xz-utils",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "zip": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["unlicense"],
                "changelog": "https://github.com/kuba--/zip/releases",
                "prereqs": [
                    "{triplet}-g++",
                    "cmake",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/kuba--/zip.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "zip",
                },
            },
        ],
    },
    "zlib": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["zlib"],
                "changelog": "https://www.zlib.net/ChangeLog.txt",
                "prereqs": [
                    "{triplet}-gcc",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/madler/zlib.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "zlib",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["zlib"],
                "changelog": "https://www.zlib.net/ChangeLog.txt",
                "checker": {
                    "type": "pacman",
                    "package": "zlib",
                },
                "installer": {
                    "type": "pacman",
                    "package": "zlib",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["zlib"],
                "changelog": "https://www.zlib.net/ChangeLog.txt",
                "checker": {
                    "type": "apt-cache",
                    "package": "zlib1g-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
}



import os

from libvgazer.command       import RunCommand
from libvgazer.exceptions    import CommandError
from libvgazer.exceptions    import InstallError
from libvgazer.platform      import GetAr
from libvgazer.platform      import GetCc
from libvgazer.platform      import GetInstallPrefix
from libvgazer.store.temp    import StoreTemp
from libvgazer.working_dir   import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)

    cc = GetCc(platformData["target"])
    ar = GetAr(platformData["target"])

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             [
              "git", "clone", "https://git.code.sf.net/p/tinyfiledialogs/code",
              "."
             ],
             verbose)
            RunCommand(
             [cc, "-c", "tinyfiledialogs.c", "-o", "tinyfiledialogs.o", "-O2",
              "-Wall", "-fPIC"],
             verbose)
            RunCommand(
             [ar, "rcs", "libtinyfiledialogs.a", "tinyfiledialogs.o"],
             verbose)
            if not os.path.exists(
             "{prefix}/include".format(prefix=installPrefix)):
                RunCommand(
                 [
                  "mkdir", "-p",
                  "{prefix}/include".format(prefix=installPrefix)
                 ],
                 verbose)
            if not os.path.exists("{prefix}/lib".format(prefix=installPrefix)):
                RunCommand(["mkdir", "-p",
                 "{prefix}/lib".format(prefix=installPrefix)], verbose)
            RunCommand(
             [
              "cp", "./tinyfiledialogs.h",
              "{prefix}/include".format(prefix=installPrefix)
             ],
             verbose)
            RunCommand(
             [
              "cp", "./libtinyfiledialogs.a",
              "{prefix}/lib".format(prefix=installPrefix)
             ],
             verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



import requests
from bs4 import BeautifulSoup

def Check():
    response = requests.get("https://ftp.gnu.org/gnu/autoconf/")
    html = response.content.decode("utf-8")
    parsedHtml = BeautifulSoup(html, "html.parser")

    links = parsedHtml.find_all("a")

    maxMajor = -1
    maxMinor = -1
    for link in links:
        if ("autoconf" in link.text and ".tar.gz" in link.text
         and ".sig" not in link.text and "-latest" not in link.text):
            version = link.text.split("-")[1].split(".tar.gz")[0]
            splitedVersion = version.split(".")
            if int(splitedVersion[0]) > maxMajor:
                maxMajor = int(splitedVersion[0])
                maxMinor = int(splitedVersion[1])
            elif (int(splitedVersion[0]) == maxMajor
             and int(splitedVersion[1]) > maxMinor):
                maxMinor = int(splitedVersion[1])

    return "{major}.{minor}".format(major=maxMajor, minor=maxMinor)



import os.path

from libvgazer.command       import GetCommandOutputUtf8
from libvgazer.exceptions    import FileNotFound
from libvgazer.exceptions    import MissingArgument
from libvgazer.exceptions    import UnexpectedOsType
from libvgazer.exceptions    import UnknownOs
from libvgazer.exceptions    import UnknownTargetArch
from libvgazer.host_detector import HostDetector
from libvgazer.utils         import FindFileInDir
from libvgazer.utils         import OneOfIsNone

def GetFilesystemType(path):
    output = GetCommandOutputUtf8(["df", "-T", path])
    lines = output.splitlines()
    if len(lines) == 1:
        raise UnexpectedOsType('Directory "' + path + '" unavailable')

    return lines[1].split()[1]

def GetHideDirectoryPrefix():
    if os.name == "posix":
        return "."
    else:
        raise UnexpectedOsType("Unexpected OS type: " + os.name)

def GetHomeDirectoryPath():
    if os.name == "posix":
        return os.path.expanduser("~")
    else:
        raise UnexpectedOsType("Unexpected OS type: " + os.name)

def GetTempDirectoryPath():
    if os.name == "posix":
        return "/tmp"
    else:
        raise UnexpectedOsType("Unexpected OS type: " + os.name)

def GetBitness(targetPlatformData):
    arch = targetPlatformData.GetArch()

    if arch == "x86_64":
        return 64
    else:
        raise UnknownTargetArch(
         "Unknown target architecture: " + arch)

def GetTriplet(targetPlatformData):
    arch = targetPlatformData.GetArch()
    os = targetPlatformData.GetOs()
    abi = targetPlatformData.GetAbi()

    triplet = arch

    if Platform.OsIsLinux(os):
        triplet += "-linux-" + abi
    elif os == "windows":
        triplet += "-w64-mingw32"
    else:
        raise UnknownOs("Unknown OS: " + os)

    return triplet

def GetGenericTriplet(targetPlatformData):
    arch = targetPlatformData.GetArch()
    os = targetPlatformData.GetOs()
    abi = targetPlatformData.GetAbi()

    triplet = arch

    if Platform.OsIsLinux(os):
        triplet += "-linux-" + abi
    elif os == "windows":
        triplet += "-w64-mingw32"
    else:
        raise UnknownOs("Unknown OS: " + os)

    return triplet

def GetInstallPrefix(platformData):
    if platformData["target"].PlatformsEqual(platformData["host"]):
        return "/usr/local"
    else:
        return ("/usr/local/" + GetTriplet(platformData["target"]))

def GetSoPrefix(platformData):
    if Platform.OsIsLinux(platformData["target"].GetOs()):
        return GetInstallPrefix(platformData) + "/lib"
    elif platformData["target"].GetOs() == "windows":
        return GetInstallPrefix(platformData) + "/bin"
    else:
        raise UnknownOs("Unknown OS: " + platformData["target"].GetOs())

def GetTripletFilenames(triplet, suffixes):
    filenames = []
    for suffix in suffixes:
        filenames.append(triplet + suffix)

    return filenames

def GetUtility(targetPlatformData, directories, suffixes, fallbackFilenames):
    triplet = GetGenericTriplet(targetPlatformData)
    vendoredTriplet = GetTriplet(targetPlatformData)

    filenames = GetTripletFilenames(triplet, suffixes)
    filenames.extend(GetTripletFilenames(vendoredTriplet, suffixes))
    try:
        return FindFileInDir(directories, filenames)
    except FileNotFound:
        return FindFileInDir(directories, fallbackFilenames)

def GetCc(targetPlatformData):
    return GetUtility(
     targetPlatformData,
     directories=["/usr/bin", "/usr/local/bin"],
     suffixes=["-gcc-posix", "-gcc"],
     fallbackFilenames=["gcc"]
    )

def GetCxx(targetPlatformData):
    return GetUtility(
     targetPlatformData,
     directories=["/usr/bin", "/usr/local/bin"],
     suffixes=["-g++-posix", "-g++"],
     fallbackFilenames=["g++"]
    )

def GetAr(targetPlatformData):
    return GetUtility(
     targetPlatformData,
     directories=["/usr/bin", "/usr/local/bin"],
     suffixes=["-ar-posix", "-gcc-ar-posix", "-ar", "-gcc-ar"],
     fallbackFilenames=["ar"]
    )

def GetArFullPath(targetPlatformData):
    return os.path.join("/usr/bin", GetAr(targetPlatformData))

def GetRanlib(targetPlatformData):
    return GetUtility(
     targetPlatformData,
     directories=["/usr/bin", "/usr/local/bin"],
     suffixes=["-ranlib-posix", "-gcc-ranlib-posix", "-ranlib", "-gcc-ranlib"],
     fallbackFilenames=["ranlib"]
    )

def GetPkgConfig(targetPlatformData):
    try:
        return GetUtility(
         targetPlatformData,
         directories=["/usr/bin", "/usr/local/bin"],
         suffixes=["-pkg-config"],
         fallbackFilenames=["pkg-config"]
        )
    except FileNotFound:
        return "pkg-config"

def GetStrip(targetPlatformData):
    return GetUtility(
     targetPlatformData,
     directories=["/usr/bin", "/usr/local/bin"],
     suffixes=["-strip"],
     fallbackFilenames=["strip"]
    )

def GetCmake(targetPlatformData):
    if os.path.isfile("/usr/bin/cmake"):
        return "/usr/bin/cmake"

    if os.path.isfile("/usr/local/bin/cmake"):
        return "/usr/local/bin/cmake"

    return "cmake"

def GetSoFilename(targetPlatformData, name):
    if Platform.OsIsLinux(targetPlatformData.GetOs()):
        return "lib" + name + ".so"
    elif targetPlatformData.GetOs() == "windows":
        return name + ".dll"
    else:
        raise UnknownOs("Unknown OS: " + targetPlatformData.GetOs())

class Platform:
    # Platforms comparing ratings
    COMP_INCOMPATIBLE = -1000
    COMP_COMPATIBLE = 0
    COMP_EQUAL = 1

    @staticmethod
    def OsIsLinux(os):
        return (os in [
            "linux", "archlinux", "oraclelinux", "debian", "steamrt"
        ])

    @staticmethod
    def GetGenericOs(os):
        if Platform.OsIsLinux(os):
            return "linux"
        elif os == "windows" or os == "any":
            return os
        else:
            raise UnknownOs("Unknown OS: " + os)

    def __init__(self, arch=None, os=None, osVersion=None, abi=None,
     suppressGenericFallback=False):
        self.isHost = OneOfIsNone(arch, os, osVersion, abi)

        if self.isHost:
            with HostDetector() as hostDetector:
                if hostDetector.OsIsUnknown():
                    raise UnexpectedOsType(hostDetector.GetErrorMsg)

                self.arch = hostDetector.GetArch()
                self.os = hostDetector.GetOs()
                self.osVersion = hostDetector.GetOsVersion()
                self.abi = hostDetector.GetAbi()

        elif not suppressGenericFallback:
            self.arch = arch
            self.os = Platform.GetGenericOs(os)
            self.osVersion = "any"
            self.abi = abi
        else:
            self.arch = arch
            self.os = os
            self.osVersion = osVersion
            self.abi = abi

    def IsHost(self):
        return self.isHost

    def GetArch(self):
        return self.arch

    def GetOs(self):
        return self.os

    def GetOsVersion(self):
        return self.osVersion

    def GetAbi(self):
        return self.abi

    def ArchsCompatible(self, platform=None, arch=None):
        if platform is None:
            if arch is None:
                raise MissingArgument("Missing value for argument 'arch'")
            comparingArch = arch
        else:
            comparingArch = platform.arch
        if self.arch == "any" or comparingArch == "any":
            return True
        if self.arch == comparingArch:
            return True

        return False

    def ArchsEqual(self, platform=None, arch=None):
        if platform is None:
            if arch is None:
                raise MissingArgument("Missing value for argument 'arch'")
            comparingArch = arch
        else:
            comparingArch = platform.arch
        if (self.arch == comparingArch and self.arch != "any" and
         comparingArch != "any"):
            return True
        return False

    def OsesCompatible(self, platform=None, os=None):
        if platform is None:
            if os is None:
                raise MissingArgument("Missing value for argument 'os'")
            comparingOs = os
        else:
            comparingOs = platform.os
        if self.os == "any" or comparingOs == "any":
            return True
        if self.os == comparingOs:
            return True
        if Platform.OsIsLinux(self.os) and comparingOs == "linux":
            return True
        return False

    def OsesEqual(self, platform=None, os=None):
        if platform is None:
            if os is None:
                raise MissingArgument("Missing value for argument 'os'")
            comparingOs = os
        else:
            comparingOs = platform.os
        if (self.os == comparingOs and self.os != "any"
         and comparingOs != "any"):
            return True
        return False

    def OsVersionsCompatible(self, platform=None, osVersion=None):
        if platform is None:
            if osVersion is None:
                raise MissingArgument("Missing value for argument 'osVersion'")
            comparingOsVersion = osVersion
        else:
            comparingOsVersion = platform.osVersion
        if self.osVersion == "any" or comparingOsVersion == "any":
            return True
        if self.osVersion == comparingOsVersion:
            return True
        return False

    def OsVersionsEqual(self, platform=None, osVersion=None):
        if platform is None:
            if osVersion is None:
                raise MissingArgument("Missing value for argument 'osVersion'")
            comparingOsVersion = osVersion
        else:
            comparingOsVersion = platform.osVersion
        if (self.osVersion == comparingOsVersion and self.osVersion != "any"
         and comparingOsVersion != "any"):
            return True
        return False

    def AbisCompatible(self, platform=None, abi=None):
        if platform is None:
            if abi is None:
                raise MissingArgument("Missing value for argument 'abi'")
            comparingAbi = abi
        else:
            comparingAbi = platform.abi
        if self.abi == "any" or comparingAbi == "any":
            return True
        if self.abi == comparingAbi:
            return True
        return False

    def AbisEqual(self, platform=None, abi=None):
        if platform is None:
            if abi is None:
                raise MissingArgument("Missing value for argument 'abi'")
            comparingAbi = abi
        else:
            comparingAbi = platform.abi
        if (self.abi == comparingAbi and self.abi != "any" and
         comparingAbi != "any"):
            return True
        return False

    def GetArchsCompareRating(self, platform=None, arch=None):
        if platform is None:
            if arch is None:
                raise MissingArgument("Missing value for argument 'arch'")
            comparingArch = arch
        else:
            comparingArch = platform.arch
        if self.ArchsEqual(arch=comparingArch):
            return Platform.COMP_EQUAL
        elif self.ArchsCompatible(arch=comparingArch):
            return Platform.COMP_COMPATIBLE
        else:
            return Platform.COMP_INCOMPATIBLE

    def GetOsesCompareRating(self, platform=None, os=None):
        if platform is None:
            if os is None:
                raise MissingArgument("Missing value for argument 'os'")
            comparingOs = os
        else:
            comparingOs = platform.os
        if self.OsesEqual(os=comparingOs):
            return Platform.COMP_EQUAL
        elif self.OsesCompatible(os=comparingOs):
            return Platform.COMP_COMPATIBLE
        else:
            return Platform.COMP_INCOMPATIBLE

    def GetOsVersionsCompareRating(self, platform=None, osVersion=None):
        if platform is None:
            if osVersion is None:
                raise MissingArgument("Missing value for argument 'osVersion'")
            comparingOsVersion = osVersion
        else:
            comparingOsVersion = platform.osVersion
        if self.OsVersionsEqual(osVersion=comparingOsVersion):
            return Platform.COMP_EQUAL
        elif self.OsVersionsCompatible(osVersion=comparingOsVersion):
            return Platform.COMP_COMPATIBLE
        else:
            return Platform.COMP_INCOMPATIBLE

    def GetAbisCompareRating(self, platform=None, abi=None):
        if platform is None:
            if abi is None:
                raise MissingArgument("Missing value for argument 'abi'")
            comparingAbi = abi
        else:
            comparingAbi = platform.abi
        if self.AbisEqual(abi=comparingAbi):
            return Platform.COMP_EQUAL
        elif self.AbisCompatible(abi=comparingAbi):
            return Platform.COMP_COMPATIBLE
        else:
            return Platform.COMP_INCOMPATIBLE

    def PlatformsEqual(self, platform):
        return (self.ArchsEqual(platform) and self.OsesEqual(platform)
         and self.OsVersionsEqual(platform) and self.AbisEqual(platform))

    def GetCompatibilityRating(self, archs, oses, osVersions, abis):
        archMaxRating = Platform.COMP_INCOMPATIBLE
        osMaxRating = Platform.COMP_INCOMPATIBLE
        osVersionMaxRating = Platform.COMP_INCOMPATIBLE
        abiMaxRating = Platform.COMP_INCOMPATIBLE
        for comparingArch in archs:
            archRating = self.GetArchsCompareRating(arch=comparingArch)
            if archRating > archMaxRating:
                archMaxRating = archRating
        for comparingOs in oses:
            osRating = self.GetOsesCompareRating(os=comparingOs)
            if osRating > osMaxRating:
                osMaxRating = osRating
        for comparingOsVersion in osVersions:
            osVersionRating = self.GetOsVersionsCompareRating(
             osVersion=comparingOsVersion)
            if osVersionRating > osVersionMaxRating:
                osVersionMaxRating = osVersionRating
        for comparingAbi in abis:
            abiRating = self.GetAbisCompareRating(abi=comparingAbi)
            if abiRating > abiMaxRating:
                abiMaxRating = abiRating

        return (archMaxRating + osMaxRating * 2 + osVersionMaxRating * 4 +
         abiMaxRating * 8)



import os

from libvgazer.exceptions import FileNotFound

def GetKeyByValue(dictionary, value):
    keys = list(dictionary.keys())
    values = list(dictionary.values())
    return keys[values.index(value)]

def FindFileInDir(directories, filenames):
    for directory in directories:
        for filename in filenames:
            if os.path.isfile(os.path.join(directory, filename)):
                return filename
    raise FileNotFound(
     "Unable to find any file from list: " + ", ".join(filenames))

def OneOfIsNone(*args):
    for arg in args:
        if arg is None:
            return True

    return False



data = {
    "libogg": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["bsd-3"],
                "changelog": "https://xiph.org/press/",
                "prereqs": [
                    "{triplet}-gcc",
                    "autoconf",
                    "automake",
                    "libtool",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.xiph.org/xiph/ogg.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "libogg",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["bsd-3"],
                "changelog": "https://xiph.org/press/",
                "checker": {
                    "type": "pacman",
                    "package": "libogg",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libogg",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["bsd-3"],
                "changelog": "https://xiph.org/press/",
                "checker": {
                    "type": "apt-cache",
                    "package": "libogg-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "libpng": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["libpng-2.0"],
                "changelog": "http://www.libpng.org/pub/png/pngnews.html",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-pkg-config",
                    "make",
                    "zlib",
                ],
                "checker": {
                    "type": "git",
                    "url": "git://git.code.sf.net/p/libpng/code",
                    "hint": r'(:?v1\.6\.\d\d$)|(:?v1\.7\.\d+$)',
                },
                "installer": {
                    "type": "custom",
                    "name": "libpng",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["bsd-3"],
                "changelog": "https://xiph.org/press/",
                "checker": {
                    "type": "pacman",
                    "package": "libpng",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libpng",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["libpng-2.0"],
                "changelog": "http://www.libpng.org/pub/png/pngnews.html",
                "checker": {
                    "type": "apt-cache",
                    "package": "libpng-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "libsir": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/aremmell/libsir/releases",
                "prereqs": [
                    "{triplet}-gcc",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/aremmell/libsir.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "libsir",
                },
            },
        ],
    },
    "libtool": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=libtool",
                "prereqs": [
                    "autoconf",
                    "automake",
                    "gcc",
                    "help2man",
                    "make",
                    "makeinfo",
                    "xz",
                ],
                "checker": {
                    "type": "git",
                    "url": "git://git.savannah.gnu.org/libtool.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "libtool",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=libtool",
                "checker": {
                    "type": "pacman",
                    "package": "libtool",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libtool",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=libtool",
                "prereqs": [
                    "autoconf",
                    "automake",
                    "gcc",
                    "help2man",
                    "make",
                    "makeinfo",
                    "patch",
                    "xz",
                ],
                "checker": {
                    "type": "git",
                    "url": "git://git.savannah.gnu.org/libtool.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "libtool",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=libtool",
                "checker": {
                    "type": "apt-cache",
                    "package": "libtool",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "libvorbis": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["bsd-3"],
                "changelog": "https://xiph.org/press/",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-pkg-config",
                    "autoconf",
                    "automake",
                    "libogg",
                    "libtool",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.xiph.org/xiph/vorbis.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "libvorbis",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["bsd-3"],
                "changelog": "https://xiph.org/press/",
                "checker": {
                    "type": "pacman",
                    "package": "libvorbis",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libvorbis",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["bsd-3"],
                "changelog": "https://xiph.org/press/",
                "checker": {
                    "type": "apt-cache",
                    "package": "libvorbis-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "libxext": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit", "x11", "hpnd", "hpnd-sv", "smlnj"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libxext/-/tags",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-pkg-config",
                    "autoconf",
                    "libtool",
                    "make",
                    "xlib",
                    "xorg-macros",
                    "xorgproto",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/xorg/lib/libxext.git",
                    "hint": r'libXext-\d\.\d(\.\d+(\.\d)?)?$',
                },
                "installer": {
                    "type": "custom",
                    "name": "libxext",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["mit", "x11", "hpnd", "hpnd-sv", "smlnj"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libxext/-/tags",
                "checker": {
                    "type": "pacman",
                    "package": "libxext",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libxext",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit", "x11", "hpnd", "hpnd-sv", "smlnj"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/libxext/-/tags",
                "checker": {
                    "type": "apt-cache",
                    "package": "libxext-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
}



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             ["git", "clone", "git://git.savannah.gnu.org/libtool.git"],
             verbose)
        clonedDir = os.path.join(tempPath, "libtool")
        with WorkingDir(clonedDir):
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("git://git.savannah.gnu.org/libtool.git")
             ],
             verbose)
            RunCommand(["./bootstrap"], verbose)
            RunCommand(["./configure", "--prefix=/usr/local"], verbose)
            RunCommand(
             ["make", "-j{cores_count}".format(cores_count=os.cpu_count())],
             verbose)
            RunCommand(["make", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



import os

from libvgazer.platform import GetTempDirectoryPath
from libvgazer.platform import GetHideDirectoryPrefix
from libvgazer.store.base import StoreBase

class StoreTemp(StoreBase):
    def __init__(self, subdirectory=None):
        tempDirPath = GetTempDirectoryPath()
        if subdirectory is None:
            subdirectory = GetHideDirectoryPrefix() + "vgazer"
        super().__init__(os.path.join(tempDirPath, subdirectory))



from libvgazer.exceptions        import UnknownVersionChecker
from libvgazer.version.apt_cache import CheckAptCache
from libvgazer.version.git       import CheckGit
from libvgazer.version.pacman    import CheckPacman
from libvgazer.version.yolk3k    import CheckYolk3k
from libvgazer.version.yum       import CheckYum

class CheckersManager:
    def __init__(self):
        self.checkFuncs = {
            "apt-cache": lambda platform, checkerData: CheckAptCache(
                checkerData["package"],
                platform.GetArch()
            ),
            "fixed": lambda platform, checkerData: checkerData["version"],
            "git": lambda platform, checkerData: CheckGit(
                checkerData["url"],
                checkerData["hint"] if "hint" in checkerData else None,
                checkerData["files"] if "files" in checkerData else None
            ),
            "pacman": lambda platform, checkerData: CheckPacman(
                checkerData["package"]
            ),
            "pypi": lambda platform, checkerData: CheckPypi(
                auth["base"],
                checkerData["package"]
            ),
            "yolk3k": lambda platform, checkerData: CheckYolk3k(
                checkerData["package"]
            ),
            "yum": lambda platform, checkerData: CheckYum(
                checkerData["package"],
                checkerData["repo"] if "repo" in checkerData else None,
                platform.GetArch()
            ),
        }

    def GetCheckFunc(self, checkerType):
        if checkerType in self.checkFuncs:
            return self.checkFuncs[checkerType]
        else:
            raise UnknownVersionChecker(
             "Unknown version checker: " + checkerType)



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.exceptions  import UnknownOs
from libvgazer.platform    import GetBitness
from libvgazer.platform    import GetCc
from libvgazer.platform    import GetCxx
from libvgazer.platform    import GetInstallPrefix
from libvgazer.platform    import GetTriplet
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def GetIcuPlatformName(osName):
    if osName == "linux":
        return "Linux/gcc"
    elif osName == "windows":
        return "MinGW"
    else:
        raise UnknownOs("Unknown generic OS: " + osName)

def Install(software, platform, platformData, verbose):
    isCrossbuild = not (
     platformData["target"].PlatformsEqual(platformData["host"])
    )
    hostBitness = GetBitness(platformData["host"])
    hostTriplet = GetTriplet(platformData["host"])
    hostCc = GetCc(platformData["host"])
    hostCxx = GetCxx(platformData["host"])
    hostOs = platformData["host"].GetOs()
    hostIcuPlatformName = GetIcuPlatformName(
     platformData["host"].GetGenericOs(hostOs))
    if isCrossbuild:
        hostInstallPrefix = "/usr/local"
        targetInstallPrefix = GetInstallPrefix(platformData)
        targetTriplet = GetTriplet(platformData["target"])
        targetCc = GetCc(platformData["target"])
        targetCxx = GetCxx(platformData["target"])
        targetOs = platformData["host"].GetOs()
        targetIcuPlatformName = GetIcuPlatformName(
         platformData["target"].GetGenericOs(targetOs))
    else:
        hostInstallPrefix = GetInstallPrefix(platformData)

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             ["git", "clone", "https://github.com/unicode-org/icu.git", "."],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://github.com/unicode-org/icu.git",
               hint=r'release-\d+-\d$')
             ],
             verbose)
            RunCommand(["mkdir", "icu4c/hostBuild"], verbose)
        hostBuildDir = os.path.join(tempPath, "icu4c/hostBuild")
        with WorkingDir(hostBuildDir):
            RunCommand(
             ["../source/runConfigureICU", hostIcuPlatformName,
              "--host=" + hostTriplet, "--prefix=" + hostInstallPrefix,
              "--disable-shared", "--enable-static=yes", "--enable-extras=no",
              "--enable-icuio=no", "--enable-layoutex=no", "--enable-tools",
              "--enable-tests=no", "--enable-samples=no",
              "--with-library-bits=" + str(hostBitness), "CC=" + hostCc,
              "CXX=" + hostCxx],
             verbose)
            RunCommand(
             ["make", "-j{cores_count}".format(cores_count=os.cpu_count())],
             verbose)
            if not isCrossbuild:
                RunCommand(["make", "install"], verbose)
        if isCrossbuild:
            with WorkingDir(tempPath):
                RunCommand(["mkdir", "icu4c/targetBuild"], verbose)
            targetBuildDir = os.path.join(tempPath, "icu4c/targetBuild")
            with WorkingDir(targetBuildDir):
                RunCommand(
                 ["../source/runConfigureICU", targetIcuPlatformName,
                  "--host=" + targetTriplet, "--prefix=" + targetInstallPrefix,
                  "--with-cross-build=" + hostBuildDir, "--disable-shared",
                  "--enable-static=yes", "--enable-extras=no",
                  "--enable-icuio=no", "--enable-layoutex=no",
                  "--enable-tools", "--enable-tests=no", "--enable-samples=no",
                  "CC=" + targetCc, "CXX=" + targetCxx],
                 verbose)
                RunCommand(
                 ["make",
                  "-j{cores_count}".format(cores_count=os.cpu_count())],
                 verbose)
                RunCommand(["make", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(software + " not installed")

    print("VGAZER:", software, "installed")



from libvgazer.command    import RunCommand
from libvgazer.exceptions import CommandError
from libvgazer.exceptions import InstallError

def InstallPacman(software, packages, verbose):
    try:
        if isinstance(packages, list):
            for package in packages:
                RunCommand(
                 [
                  "pacman", "--noconfirm", "--disable-download-timeout", "-S",
                  package
                 ],
                 verbose
                )
        else:
            RunCommand(
             [
              "pacman", "--noconfirm", "--disable-download-timeout", "-S",
              packages
             ],
             verbose
            )
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software}  not installed".format(software=software))

    print("VGAZER:", software, "installed")



class OsRelease:
    def __init__(self):
        self.entries = {}

        with open("/etc/os-release", "r") as osReleaseFile:
            data = osReleaseFile.read()
        lines = data.splitlines()
        for line in lines:
            if "=" in line:
                kv = line.split("=")
                self.entries[kv[0]] = kv[1]

    def __enter__(self):
        return self

    def __exit__(self, etype, value, traceback):
        pass

    def GetEntry(self, key):
        if key in self.entries:
            return self.entries[key].strip("\"")



import os
from pathlib import Path

class WorkingDir:
    """Context manager for changing the current working directory"""
    def __init__(self, newPath, verbose=True):
        self.newPath = os.path.expanduser(newPath)
        self.verbose = verbose

    def __enter__(self):
        self.savedPath = os.getcwd()
        Path(self.newPath).mkdir(parents=True, exist_ok=True)
        if self.verbose:
            print("VGAZER: Entering to directory:", self.newPath)
        os.chdir(self.newPath)

    def __exit__(self, etype, value, traceback):
        if self.verbose:
            print("VGAZER: Returning to directory:", self.savedPath)
        os.chdir(self.savedPath)



data = {
    "alsa-lib": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["lgpl-2.1"],
                "changelog": "https://www.alsa-project.org/wiki/Main_Page_News",
                "prereqs": [
                    "{triplet}-gcc",
                    "automake",
                    "libtool",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/alsa-project/alsa-lib.git",
                    "hint": r'v1\.2\.\d\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "alsa-lib",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["lgpl-2.1"],
                "changelog": "https://www.alsa-project.org/wiki/Main_Page_News",
                "checker": {
                    "type": "pacman",
                    "package": "alsa-lib",
                },
                "installer": {
                    "type": "pacman",
                    "package": "alsa-lib",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["lgpl-2.1"],
                "changelog": "https://www.alsa-project.org/wiki/Main_Page_News",
                "checker": {
                    "type": "apt-cache",
                    "package": "libasound2-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "autoconf": {
        "platform": "host",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/autoconf/NEWS",
                "prereqs": [
                    "m4",
                    "make",
                    "wget",
                ],
                # Using custom checker and installer because we dont have
                # automake for generating .configure when building autoconf
                # from git source
                "checker": {
                    "type": "custom",
                    "name": "autoconf",
                },
                "installer": {
                    "type": "custom",
                    "name": "autoconf",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/autoconf/NEWS",
                "checker": {
                    "type": "pacman",
                    "package": "autoconf",
                },
                "installer": {
                    "type": "pacman",
                    "package": "autoconf",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/autoconf/NEWS",
                "prereqs": [
                    "m4",
                    "make",
                    "perl-data-dumper",
                    "wget",
                ],
                "checker": {
                    "type": "custom",
                    "name": "autoconf",
                },
                "installer": {
                    "type": "custom",
                    "name": "autoconf",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/autoconf/NEWS",
                "checker": {
                    "type": "apt-cache",
                    "package": "autoconf",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "automake": {
        "platform": "host",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/automake/NEWS",
                "prereqs": [
                    "autoconf",
                    "gcc",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://git.savannah.gnu.org/git/automake.git",
                    "hint": r'v\d\.\d+\w?(\.\d+)?$',
                },
                "installer": {
                    "type": "custom",
                    "name": "automake",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/automake/NEWS",
                "checker": {
                    "type": "pacman",
                    "package": "automake",
                },
                "installer": {
                    "type": "pacman",
                    "package": "automake",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                # autoconf >=2.70 required for building xlib
                # Oracle Linux 7 has only 2.69
                # automake depends on autoconf
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/automake/NEWS",
                "prereqs": [
                    "autoconf",
                    "gcc",
                    "make",
                    "perl-thread-queue",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://git.savannah.gnu.org/git/automake.git",
                    "hint": r'v\d\.\d+\w?(\.\d+)?$',
                },
                "installer": {
                    "type": "custom",
                    "name": "automake",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/automake/NEWS",
                "checker": {
                    "type": "apt-cache",
                    "package": "automake",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "autopoint": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/gettext/ChangeLog",
                "checker": {
                    "type": "pacman",
                    "package": "gettext",
                },
                "installer": {
                    "type": "pacman",
                    "package": "gettext",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/gettext/ChangeLog",
                "checker": {
                    "type": "yum",
                    "package": "gettext-devel",
                },
                "installer": {
                    "type": "yum",
                    "package": "gettext-devel",
                },
            },
            # Temporary stub
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://fossies.org/linux/gettext/ChangeLog",
                "checker": {
                    "type": "apt-cache",
                    "package": "autopoint",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "cfgpath": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["unlicense"],
                "changelog": "https://github.com/Malvineous/cfgpath/commits/master/",
                "checker": {
                    "type": "git",
                    "url": "https://github.com/Malvineous/cfgpath.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "cfgpath",
                },
            },
        ],
    },
    "cjson": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/DaveGamble/cJSON/blob/master/CHANGELOG.md",
                "prereqs": [
                    "{triplet}-gcc",
                    "make",
                    "cmake",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/DaveGamble/cJSON.git",
                    "hint": r'v1\.7\.\d\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "cjson",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "https://github.com/DaveGamble/cJSON/blob/master/CHANGELOG.md",
                "checker": {
                    "type": "pacman",
                    "package": "cjson",
                },
                "installer": {
                    "type": "pacman",
                    "package": "cjson",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "https://github.com/DaveGamble/cJSON/blob/master/CHANGELOG.md",
                "checker": {
                    "type": "apt-cache",
                    "package": "libcjson-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "clang": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://clang.llvm.org/extra/ReleaseNotes.html#what-s-new-in-extra-clang-tools-release",
                "checker": {
                    "type": "pacman",
                    "package": "clang",
                },
                "installer": {
                    "type": "pacman",
                    "package": "clang",
                },
            },
        ],
    },
    "clang-tidy": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://clang.llvm.org/extra/ReleaseNotes.html#improvements-to-clang-tidy",
                "prereqs": ["clang"],
                "checker": {
                    "type": "pacman",
                    "package": "clang",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "cmake": {
        "platform": "host",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://cmake.org/cmake/help/latest/release/index.html",
                "prereqs": [
                    "g++",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.kitware.com/cmake/cmake.git",
                    "hint": r'v3\.\d\d\.\d+',
                },
                "installer": {
                    "type": "custom",
                    "name": "cmake",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://cmake.org/cmake/help/latest/release/index.html",
                "checker": {
                    "type": "pacman",
                    "package": "cmake",
                },
                "installer": {
                    "type": "pacman",
                    "package": "cmake",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://cmake.org/cmake/help/latest/release/index.html",
                "checker": {
                    "type": "apt-cache",
                    "package": "cmake",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "cmake_barebones": {
        "platform": "host",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["cc0"],
                "changelog": "https://github.com/edomin/cmake_barebones/commits/main/",
                "checker": {
                    "type": "git",
                    "url": "https://github.com/edomin/cmake_barebones.git"
                },
                "installer": {
                    "type": "custom",
                    "name": "cmake_barebones",
                },
            },
        ],
    },
    "cmocka": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["apache-2.0"],
                "changelog": "https://cmocka.org/#news",
                "prereqs": [
                    "{triplet}-gcc",
                    "make",
                    "cmake",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://git.cryptomilk.org/projects/cmocka.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "cmocka",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["apache-2.0"],
                "changelog": "https://cmocka.org/#news",
                "checker": {
                    "type": "pacman",
                    "package": "cmocka",
                },
                "installer": {
                    "type": "pacman",
                    "package": "cmocka",
                },
            },
        ],
    },
    "codecoverage": {
        "platform": "host",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["bsd-3"],
                "changelog": "https://github.com/bilke/cmake-modules/blob/master/CodeCoverage.cmake#L29",
                "checker": {
                    "type": "git",
                    "url": "https://github.com/bilke/cmake-modules.git",
                    "files": ["CodeCoverage.cmake"],
                    "hint": r'\d{4}-\d{2}-\d{2}',
                },
                "installer": {
                    "type": "custom",
                    "name": "codecoverage",
                },
            },
        ],
    },
    "cwalk": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/likle/cwalk/releases",
                "prereqs": [
                    "{triplet}-gcc",
                    "make",
                    "cmake",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/likle/cwalk",
                },
                "installer": {
                    "type": "custom",
                    "name": "cwalk",
                },
            },
        ],
    },
    "dr_wav": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["unlicense"],
                "changelog": "https://github.com/mackron/dr_libs/blob/master/dr_wav.h#L8351",
                "checker": {
                    "type": "git",
                    "url": "https://github.com/mackron/dr_libs.git",
                    "files": ["dr_wav.h"],
                    "hint": r'v\d\.\d+\.\d+',
                },
                "installer": {
                    "type": "custom",
                    "name": "dr_wav",
                },
            },
        ],
    },
    "expat": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/libexpat/libexpat/blob/master/expat/Changes",
                "prereqs": [
                    "{triplet}-g++",
                    "{triplet}-gcc",
                    "autoconf",
                    "automake",
                    "libtool",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/libexpat/libexpat.git",
                    "hint": r'R_\d_\d_\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "expat",
                },
            },
        ],
    },
}



from libvgazer.software.a_f       import data as a_f
from libvgazer.software.g_i       import data as g_i
from libvgazer.software.j_libm    import data as j_libm
from libvgazer.software.libo_libz import data as libo_libz
from libvgazer.software.lin_o     import data as lin_o
from libvgazer.software.p_s       import data as p_s
from libvgazer.software.t_xc      import data as t_xc
from libvgazer.software.xd_z      import data as xd_z

class SoftwareData:
    def __init__(self, customData={}):
        self.data = {
            **a_f,
            **g_i,
            **j_libm,
            **libo_libz,
            **lin_o,
            **p_s,
            **t_xc,
            **xd_z,
            **customData,
        }

    def AddData(self, customData):
        self.data = {**self.data, **customData}

    def GetData(self):
        return self.data



data = {
    "ketopt": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/attractivechaos/klib/commits/master/ketopt.h",
                "checker": {
                    "type": "git",
                    "url": "https://github.com/attractivechaos/klib",
                    "files": ["ketopt.h"],
                },
                "installer": {
                    "type": "custom",
                    "name": "ketopt"
                },
            },
        ],
    },
    "libclipboard": {
        "platform": "target",
        "projects": [
            {
                "arch": ["any"],
                "os": ["linux"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/jtanx/libclipboard/releases",
                "prereqs": [
                    "{triplet}-g++",
                    "{triplet}-pkg-config",
                    "cmake",
                    "make",
                    "xcb",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/jtanx/libclipboard.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "libclipboard",
                },
            },
            {
                "arch": ["any"],
                "os": ["windows"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/jtanx/libclipboard/releases",
                "prereqs": [
                    "{triplet}-g++",
                    "{triplet}-pkg-config",
                    "cmake",
                    "make",
                ],
                "checker": {
                    "type": "github",
                    "user": "jtanx",
                    "repo": "libclipboard",
                },
                "installer": {
                    "type": "custom",
                    "name": "libclipboard",
                },
            },
        ],
    },
    "libffi": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/libffi/libffi/releases",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-g++",
                    "autoconf",
                    "automake",
                    "libtool",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/libffi/libffi.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "libffi",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "https://github.com/libffi/libffi/releases",
                "checker": {
                    "type": "pacman",
                    "package": "libffi",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libffi",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "https://github.com/libffi/libffi/releases",
                "checker": {
                    "type": "apt-cache",
                    "package": "libffi-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    # libintl under GNU LGPL 2.1
    # libintl-lite under version 1 of the Boost Software License
    "libintl": {
        "platform": "target",
        # TODO
        "projects": [
            {
                "arch": ["any"],
                "os": ["windows"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["bsl-1.0"],
                "changelog": "https://github.com/j-jorge/libintl-lite/commits/master/",
                "checker": {
                    "type": "git",
                    "url": "https://github.com/j-jorge/libintl-lite.git",
                    "files": ["CMakeLists.txt"],
                    "hint": r'LibIntl VERSION \d\.\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "libintl-lite",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["lgpl-2.1"],
                "changelog": "https://fossies.org/linux/gettext/ChangeLog",
                "checker": {
                    "type": "apt-cache",
                    "package": "libc6-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "liblzma": {
        "platform": "target",
        "projects": [
            {
                "arch": ["any"],
                "os": ["linux"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["pd"],
                "changelog": "https://github.com/tukaani-project/xz/blob/master/NEWS",
                "prereqs": [
                    "{triplet}-gcc",
                    "autoconf",
                    "automake",
                    "autopoint",
                    "libtool",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/tukaani-project/xz.git",
                    "hint": r'v\d\.\d.\d+$',
                },
                "installer": {
                    "type": "custom",
                    "name": "liblzma",
                },
            },
            {
                "arch": ["any"],
                "os": ["windows"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["pd"],
                "changelog": "https://github.com/tukaani-project/xz/blob/master/NEWS",
                "prereqs": [
                    "{triplet}-gcc",
                    "autoconf",
                    "automake",
                    "autopoint",
                    "libtool",
                    "make",
                    # "libintl",
                ],
                "checker": {
                    "type": "sourceforge",
                    "project": "lzmautils",
                },
                "installer": {
                    "type": "custom",
                    "name": "liblzma",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["pd"],
                "changelog": "https://github.com/tukaani-project/xz/blob/master/NEWS",
                "prereqs": [
                    "xz",
                ],
                "checker": {
                    "type": "pacman",
                    "package": "xz",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["pd"],
                "changelog": "https://github.com/tukaani-project/xz/blob/master/NEWS",
                "checker": {
                    "type": "apt-cache",
                    "package": "liblzma-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
}



import os

def PrintMessage(varName):
    if varName in os.environ:
        print("VGAZER: Changed environment variable:", varName)
        print("VGAZER: New value:", os.environ[varName])
    else:
        print("VGAZER: Removed environment variable:", varName)

def SetEnvVar(name, value):
    os.environ[name] = value
    PrintMessage(name)

class EnvVar:
    def __init__(self, name, value):
        self.name = name
        self.newValue = value

    def __enter__(self):
        if self.name not in os.environ:
            self.oldValue = None
        else:
            self.oldValue = os.environ[self.name]

        os.environ[self.name] = self.newValue
        PrintMessage(self.name)

    def __exit__(self, etype, value, traceback):
        if self.oldValue is None:
            os.environ.pop(self.name)
        else:
            os.environ[self.name] = self.oldValue
        PrintMessage(self.name)



data = {
    "apache-2.0": {
        "name": "Apache-2.0",
        "fullName": "Apache License 2.0",
        "rules": {
            "appendNoticeFileIfExists": True,
            "distributeOriginalLicenseText": True,
        }
    },
    "beerware": {
        "name": "Beerware",
        "fullName": "Beerware",
        "rules": {}
    },
    "bsd-1": {
        "name": "BSD 1-clause",
        "fullName": "BSD 1-Clause License",
        "rules": {}
    },
    "bsd-2": {
        "name": "BSD 2-clause",
        "fullName": "Simplified BSD License",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "bsd-3": {
        "name": "BSD 3-clause",
        "fullName": "Modified BSD License",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "bsl-1.0": {
        "name": "BSD 3-clause",
        "fullName": "Modified BSD License",
        "rules": {}
    },
    "bzip2-1.0.6": {
        "name": "bzip2-1.0.6",
        "fullName": "bzip2 and libbzip2 License v1.0.6",
        "rules": {}
    },
    "cc0": {
        "name": "CC0",
        "fullName": "CC0",
        "rules": {}
    },
    "ftl": {
        "name": "FTL",
        "fullName": "MIT License",
        "rules": {
            "mentionSource": True,
        }
    },
    "gpl-2-lsn": {
        "name": "GPL-2.0 WITH Linux-syscall-note",
        "fullName":
            "GNU General Public License Version 2.0 WITH Linux-syscall-note",
        "rules": {}
    },
    "hpnd": {
        "name": "HPND",
        "fullName": "Historical Permission Notice and Disclaimer",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "hpnd-sv": {
        "name": "HPND-sell-variant",
        "fullName":
            "Historical Permission Notice and Disclaimer - sell variant",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "icu": {
        "name": "ICU",
        "fullName": "ICU License",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "ijg": {
        "name": "IJG",
        "fullName": "Independent JPEG Group License",
        "rules": {
            "mentionSource": True,
        }
    },
    "isc": {
        "name": "ISC",
        "fullName": "ISC license",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "lgpl-2": {
        "name": "LGPL-2",
        "fullName": "GNU Lesser General Public License Version 2.0",
        "rules": {
            "noStaticLink": True,
        }
    },
    "lgpl-2.1": {
        "name": "LGPL-2.1",
        "fullName": "GNU Lesser General Public License Version 2.1",
        "rules": {
            "noStaticLink": True,
        }
    },
    "lgpl-2.1+": {
        "name": "LGPL-2.1 or later",
        "fullName": "GNU Lesser General Public License Version 2.1 or later",
        "rules": {
            "noStaticLink": True,
        }
    },
    "lgpl-3": {
        "name": "LGPL-3",
        "fullName": "GNU Lesser General Public License Version 3",
        "rules": {
            "noStaticLink": True,
        }
    },
    "libpng-2.0": {
        "name": "libpng-2.0",
        "fullName": "PNG Reference Library License version 2",
        "rules": {}
    },
    "libtiff": {
        "name": "libtiff",
        "fullName": "libtiff License",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "mit": {
        "name": "MIT",
        "fullName": "MIT License",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "naist": {
        "name": "NAIST-2003",
        "fullName": "Nara Institute License 2003",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "pd": {
        "name": "Public Domain",
        "fullName": "Public Domain",
        "rules": {}
    },
    "sgi-b-1.1": {
        "name": "SGI-B-1.1",
        "fullName": "SGI Free Software License B v1.1",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "sgi-b-2.0": {
        "name": "SGI-B-2.0",
        "fullName": "SGI Free Software License B v2.0",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "smlnj": {
        "name": "SML/NJ",
        "fullName":
            "Standard ML of New Jersey Copyright Notice, License And "
            "Disclaimer",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "unicode": {
        "name": "Unicode",
        "fullName":
            "Unicode, Inc. License Agreement - Data Files And Software",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "unlicense": {
        "name": "Unlicense",
        "fullName": "Unlicense",
        "rules": {}
    },
    "x11": {
        "name": "X11",
        "fullName": "X11 License",
        "rules": {
            "distributeOriginalLicenseText": True,
        }
    },
    "zlib": {
        "name": "Zlib",
        "fullName":
            "Unicode, Inc. License Agreement - Data Files And Software",
        "rules": {}
    },
}

class LicensesData:
    def __init__(self, customData={}):
        self.data = {**data, **customData}

    def AddData(self, customData):
        self.data = {**self.data, **customData}

    def GetData(self):
        return self.data



import os
import platform

from libvgazer.exceptions import OsDataNotFound
from libvgazer.exceptions import ReleaseDataNotFound
from libvgazer.exceptions import UnknownOsVersion
from libvgazer.os_release import OsRelease

class HostDetector:
    @staticmethod
    def GetLinuxOs():
        with OsRelease() as osRelease:
            try:
                return {
                    "arch":   "archlinux",
                    "ol":     "oraclelinux",
                    "debian": "debian",
                }[osRelease.GetEntry("ID")]
            except KeyError:
                raise OsDataNotFound(
                 "Unable to find data of host OS: " + os.name)

    @staticmethod
    def GetOracleLinuxVersion():
        with OsRelease() as osRelease:
            try:
                return osRelease.GetEntry("VERSION").strip("\"").split(".")[0]
            except KeyError:
                raise ReleaseDataNotFound(
                 "Unable to find data of Debian version: " + os.name)

    @staticmethod
    def GetDebianVersion():
        with OsRelease() as osRelease:
            try:
                return osRelease.GetEntry("VERSION").split("(")[1][:-2:]
            except KeyError:
                raise ReleaseDataNotFound(
                 "Unable to find data of Debian version: " + os.name)

    @staticmethod
    def GetSteamRtVersion():
        # TODO
        # May be reference - https://gitlab.steamos.cloud/steamrt/steam-runtime-tools/-/blob/main/docs/distro-assumptions.md#environment-variables
        raise UnknownOsVersion(
                 "Unable to detect version of Steam Runtime")

    def __init__(self):
        self.unknownOs = False
        self.errorMsg = "No error"
        self.arch = platform.machine()
        osType = platform.system()
        if osType == "Linux":
            # TODO steamrt not being detected by HostDetector.GetLinuxOs()
            self.os = HostDetector.GetLinuxOs()
            if self.os == "archlinux":
                self.osVersion = "latest"
                self.abi = "gnu"
            if self.os == "oraclelinux":
                self.osVersion = HostDetector.GetOracleLinuxVersion()
                self.abi = "gnu"
            if self.os == "debian":
                self.osVersion = HostDetector.GetDebianVersion()
                self.abi = "gnu"
            if self.os == "steamrt":
                self.osVersion = HostDetector.GetSteamRtVersion()
                self.abi = "gnu"
        else:
            self.unknownOs = True
            self.errorMsg = "Unexpected OS type: " + osType

    def __enter__(self):
        return self

    def __exit__(self, etype, value, traceback):
        pass

    def OsIsUnknown(self):
        return self.unknownOs

    def GetErrorMsg(self):
        return self.errorMsg

    def GetArch(self):
        return self.arch

    def GetOs(self):
        return self.os

    def GetOsVersion(self):
        return self.osVersion

    def GetAbi(self):
        return self.abi



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             ["git", "clone", "https://github.com/ninja-build/ninja.git", "."],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://github.com/ninja-build/ninja.git",
               hint=r'v1\.\d\d\.\d+')
             ],
             verbose)
            RunCommand(
             ["./configure.py", "--bootstrap", "--verbose"],
             verbose)
            if not os.path.exists("/usr/local/bin"):
                RunCommand(["mkdir", "-p", "/usr/local/bin"], verbose)
            RunCommand(["cp", "./ninja", "/usr/local/bin/ninja"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



from libvgazer.command    import RunCommand
from libvgazer.exceptions import CommandError
from libvgazer.exceptions import InstallError

def InstallYum(software, packages, repo, postInstallCommands, verbose):
    cmd = ["yum", "-y", "install"]
    if repo is not None:
        cmd.append("--enablerepo={repo}".format(repo=repo))

    try:
        if isinstance(packages, list):
            for package in packages:
                cmd.append(package)
        else:
            cmd.append(packages)
        RunCommand(cmd, verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software}  not installed".format(software=software))

    if postInstallCommands is not None:
        for command in postInstallCommands:
            RunCommand(command, verbose)

    print("VGAZER:", software, "installed")



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.platform    import GetInstallPrefix
from libvgazer.store.temp  import StoreTemp
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             ["git", "clone", "https://github.com/edomin/cmake_barebones.git"],
             verbose)
        srcDir = os.path.join(tempPath, "cmake_barebones")
        with WorkingDir(srcDir):
            RunCommand(
             ["./install.sh",
              "--prefix={prefix}".format(prefix=installPrefix)],
             verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(software + " not installed")

    print("VGAZER:", software, "installed")



import os
import re

from multimethod import multimethod

from libvgazer.checkers_manager   import CheckersManager
from libvgazer.exceptions         import CompatibleProjectNotFound
from libvgazer.exceptions         import InstallError
from libvgazer.exceptions         import UnknownSoftware
from libvgazer.install.custom     import InstallCustom
from libvgazer.installers_manager import InstallersManager
from libvgazer.platform           import GetGenericTriplet
from libvgazer.platform           import Platform
from libvgazer.software           import SoftwareData
from libvgazer.version.custom     import VersionCustom

class Vgazer:
    def __init__(self, arch=None, os=None, osVersion=None, abi=None,
     customCheckers={}, customInstallers={}, customSoftwareData={},
     supportOnly=False):
        if not supportOnly:
            self.versionCustom = VersionCustom(customCheckers)
            self.checkersManager = CheckersManager()
            self.installersManager = InstallersManager()
            self.installCustom = InstallCustom(customInstallers)
            self.installedSoftware = []
        self.softwareData = SoftwareData(customSoftwareData)
        self.platform = {
            "host":   Platform(),
            "target": Platform(arch, os, osVersion, abi),
        }

    def __del__(self):
        if (hasattr(self, "installedSoftware")
         and len(self.installedSoftware) != 0):
            print("VGAZER: Installed software (widh dependencies):")
            for software in self.installedSoftware:
                print("VGAZER:   {software}".format(software=software))

    def GetHostPlatform(self):
        return self.platform["host"]

    def GetTargetPlatform(self):
        return self.platform["target"]

    def GetSoftwareData(self):
        return self.softwareData

    @multimethod
    def ChooseProject(self, projects: list, platformData: Platform):
        maxRatingProject = None
        maxRating = Platform.COMP_INCOMPATIBLE
        for project in projects:
            projectRating = platformData.GetCompatibilityRating(project["arch"],
             project["os"], project["osVersion"], project["abi"])
            if projectRating > maxRating:
                maxRating = projectRating
                maxRatingProject = project
        if maxRating >= Platform.COMP_COMPATIBLE:
            return maxRatingProject
        else:
            return None

    @multimethod
    def ChooseProject(self, softwareData: dict):
        return self.ChooseProject(softwareData["projects"],
         self.platform[softwareData["platform"]])

    def SearchFallbackProject(self, projects):
        for project in projects:
            if "fallback" in project:
                return project

        return None

    def NormalizeVersion(self, version):
        normalized = version.split(":")[1] if ":" in version else version
        normalized = re.sub(r'^\D(?:\D*[a-z])?[a-zA-Z_]?(?:\d*\-)?' , "" ,
         normalized)
        normalized = re.sub(r'\.\D*$' , "" , normalized)
        splitted = re.split(r'[\.\_\-\:\s]', normalized)

        if len(splitted) > 4:
            return ".".join(splitted[0:4]) + "".join(splitted[4:])

        return ".".join(splitted)

    def UseChecker(self, software, checker):
        softwareData = self.softwareData.GetData()
        softwarePlatform = softwareData[software]["platform"]

        if checker["type"] == "custom":
            return self.versionCustom.Check(checker["name"])
        else:
            checkFunc = self.checkersManager.GetCheckFunc(checker["type"])
            return checkFunc(self.platform[softwarePlatform], checker)

    def CheckVersion(self, software):
        softwareData = self.softwareData.GetData()
        if software not in softwareData:
            raise UnknownSoftware("Unknown software: {software}".format(
             software=software))

        project = self.ChooseProject(softwareData[software])
        if project is None:
            raise CompatibleProjectNotFound(
             "Unable to find compatible project for software: "
             "{software}".format(software=software))

        checker = project["checker"]

        return self.NormalizeVersion(self.UseChecker(software, checker))

    def VersionsList(self, softwareList):
        result = []
        for software in softwareList:
            result.append({
                "software": software,
                "version": self.CheckVersion(software)
            })
        return result

    def UseInstaller(self, software, installer, verbose):
        softwareData = self.softwareData.GetData()
        softwarePlatform = softwareData[software]["platform"]

        if software in self.installedSoftware:
            return

        try:
            if installer["type"] == "custom":
                self.installCustom.Install(software, installer["name"],
                 softwarePlatform, self.platform, verbose)
            else:
                installFunc = self.installersManager.GetInstallFunc(
                 installer["type"])
                installFunc(software, self.platform, installer, verbose)
            self.installedSoftware.append(software)
        except InstallError as installError:
            raise installError

    def InstallProject(self, software, project, verbose):
        prereqs = project["prereqs"] if "prereqs" in project else []

        for prereq in prereqs:
            prereq = prereq.format(
             triplet=GetGenericTriplet(self.platform["target"]))
            if prereq not in self.installedSoftware:
                self.Install(prereq, verbose)

        installer = project["installer"]

        try:
            self.UseInstaller(software, installer, verbose)
        except InstallError as installError:
            fallbackProject = self.SearchFallbackProject(softwareProjects);
            if (fallbackProject is not None and fallbackProject is not project):
                print(
                 "VGAZER: Something went wrong. Starting fallback installation "
                 "steps"
                )
                self.InstallProject(software, fallbackProject, verbose)
            else:
                raise installError

    def Install(self, software, verbose=False):
        if software in self.installedSoftware:
            return

        softwareData = self.softwareData.GetData()
        if software not in softwareData:
            raise UnknownSoftware("Unknown software: " + software)

        project = self.ChooseProject(softwareData[software])
        if project is None:
            raise CompatibleProjectNotFound(
             "Unable to find compatible project for software: " + software)

        self.InstallProject(software, project, verbose)

    def InstallListFromFile(self, filename, verbose=False):
        with open(filename) as file:
            for line in file:
                line = line.rstrip("\n")
                if os.path.isfile(line):
                    self.InstallListFromFile(line, verbose)
                else:
                    self.Install(line, verbose)

    def InstallList(self, softwareList, verbose=False):
        for software in softwareList:
            if os.path.isfile(software):
                self.InstallListFromFile(software, verbose)
            else:
                self.Install(software, verbose)



from libvgazer.exceptions import UnknownTargetArch
from libvgazer.platform   import GetAr
from libvgazer.platform   import GetCc
from libvgazer.platform   import GetCmake
from libvgazer.platform   import GetCxx
from libvgazer.platform   import GetPkgConfig
from libvgazer.platform   import GetStrip
from libvgazer.platform   import Platform
from libvgazer.store.temp import StoreTemp

class ConfigMeson():
    def __init__(self, platformData):
        print("VGAZER: Generating Meson cross-build definition file...")

        storeTemp = StoreTemp(".vgazer/meson")
        self.crossFileName = storeTemp.GetDirectoryFilePath("cross-file")
        storeTemp.ResolveDirectory()
        platformData = platformData

        ar = GetAr(platformData["target"])
        cc = GetCc(platformData["target"])
        cxx = GetCxx(platformData["target"])
        pkgConfig = GetPkgConfig(platformData["target"])
        strip = GetStrip(platformData["target"])
        cmake = GetCmake(platformData["target"])
        genericOs = Platform.GetGenericOs(platformData["target"].GetOs())
        if platformData["target"].GetArch() == "x86_64":
            mesonCpuFamily = "x86_64"
            mesonCpu = "x86_64"
        else:
            raise UnknownTargetArch(
             "Unknown target architecture: {arch}".format(
              arch=platformData["target"].GetArch())
            )

        storeTemp.DirectoryWriteTextFile("cross-file",
         "[binaries]\n"
         "c = '{cc}'\n"
         "cpp = '{cxx}'\n"
         "ar = '{ar}'\n"
         "strip = '{strip}'\n"
         "pkgconfig = '{pkgConfig}'\n"
         "cmake = '{cmake}'\n"
         "\n"
         "[host_machine]\n"
         "system = '{genericOs}'\n"
         "cpu_family = '{mesonCpuFamily}'\n"
         "cpu = '{mesonCpu}'\n"
         "endian = 'little'".format(ar=ar, cc=cc, cmake=cmake, cxx=cxx,
          pkgConfig=pkgConfig, strip=strip, genericOs=genericOs,
          mesonCpuFamily=mesonCpuFamily, mesonCpu=mesonCpu)
        )

    def __enter__(self):
        return self.crossFileName

    def __exit__(self, etype, value, traceback):
        pass



import os

from libvgazer.command      import RunCommand
from libvgazer.env_vars     import EnvVar
from libvgazer.config.meson import ConfigMeson
from libvgazer.exceptions   import CommandError
from libvgazer.exceptions   import InstallError
from libvgazer.platform     import GetInstallPrefix
from libvgazer.store.temp   import StoreTemp
from libvgazer.version.git  import GetLastTag
from libvgazer.working_dir  import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    if "PKG_CONFIG_LIBDIR" in os.environ:
        pkgConfigLibdir =
         "/usr/local/lib/pkgconfig:/usr/local/lib64/pkgconfig:{old}".format(
          old=os.environ["PKG_CONFIG_LIBDIR"])
    else:
        pkgConfigLibdir = "/usr/local/lib/pkgconfig:/usr/local/lib64/pkgconfig"

    try:
        with WorkingDir(tempPath), EnvVar("PKG_CONFIG_LIBDIR", pkgConfigLibdir), ConfigMeson(platformData) as conf:
            RunCommand(
             [
              "git", "clone",
              "https://gitlab.freedesktop.org/wayland/wayland.git", "."
             ],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://gitlab.freedesktop.org/wayland/wayland.git",
               hint=r'1\.\d\d.\d+')
             ],
             verbose)
            RunCommand(
             [
              "meson", "setup", "build/",
              "--prefix={prefix}".format(prefix=installPrefix), "--cross-file",
              conf, "-Dscanner=false", "-Dtests=false",
              "-Ddocumentation=false", "-Ddtd_validation=false"
             ],
             verbose)
            RunCommand(["ninja", "-C", "build/"], verbose)
            RunCommand(["ninja", "-C", "build/", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



import os

from libvgazer.command     import RunCommand
from libvgazer.env_vars    import EnvVar
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.platform    import GetInstallPrefix
from libvgazer.platform    import GetTriplet
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)
    targetTriplet = GetTriplet(platformData["target"])

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    aclocal = "aclocal -I {prefix}/share/aclocal".format(prefix=installPrefix)

    try:
        with WorkingDir(tempPath), EnvVar("ACLOCAL", aclocal):
            RunCommand(
             [
              "git", "clone",
              "https://gitlab.freedesktop.org/xorg/lib/libxtrans.git", "."
             ],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag(
               "https://gitlab.freedesktop.org/xorg/lib/libxtrans.git",
               hint=r'xtrans-\d\.\d\.\d')
             ],
             verbose)
            RunCommand(
             [
              "./autogen.sh", "--host={triplet}".format(triplet=targetTriplet),
              "--prefix={prefix}".format(prefix=installPrefix),
              "PKG_CONFIG_PATH={prefix}/lib/pkgconfig:"
              "{prefix}/share/pkgconfig".format(prefix=installPrefix)
             ],
             verbose)
            RunCommand(
             ["make", "-j{cores_count}".format(cores_count=os.cpu_count())],
             verbose)
            RunCommand(["make", "install"], verbose)

    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             [
              "git", "clone", "https://gitlab.kitware.com/cmake/cmake.git", "."
             ],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://gitlab.kitware.com/cmake/cmake.git",
               hint=r'v3\.\d\d\.\d+')
             ],
             verbose)
            RunCommand(
             [
              "./bootstrap",
              "--parallel={cores_count}".format(cores_count=os.cpu_count()),
              "--",
              "-DCMAKE_USE_OPENSSL=OFF",
             ],
             verbose)
            RunCommand(["make"], verbose)
            RunCommand(["make", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(software + " not installed")

    print("VGAZER:", software, "installed")



data = {
    "llvm": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://llvm.org/docs/ReleaseNotes.html",
                "checker": {
                    "type": "pacman",
                    "package": "llvm",
                },
                "installer": {
                    "type": "pacman",
                    "package": "llvm",
                },
            },
        ],
    },
    "luajit": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "http://luajit.org/status.html",
                "prereqs": [
                    "{triplet}-binutils",
                    "{triplet}-gcc",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://luajit.org/git/luajit.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "luajit",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "http://luajit.org/status.html",
                "checker": {
                    "type": "pacman",
                    "package": "luajit",
                },
                "installer": {
                    "type": "pacman",
                    "package": "luajit",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "http://luajit.org/status.html",
                "checker": {
                    "type": "apt-cache",
                    "package": "libluajit-5.1-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "lwrb": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/MaJerle/lwrb/blob/develop/CHANGELOG.md",
                "prereqs": [
                    "{triplet}-gcc",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/MaJerle/lwrb.git",
                    "hint": r'v\d\.\d\.\d$',
                },
                "installer": {
                    "type": "custom",
                    "name": "lwrb",
                },
            },
        ],
    },
    "m4": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "http://savannah.gnu.org/news/?group=m4",
                "checker": {
                    "type": "pacman",
                    "package": "m4",
                },
                "installer": {
                    "type": "pacman",
                    "package": "m4",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "http://savannah.gnu.org/news/?group=m4",
                "checker": {
                    "type": "yum",
                    "package": "m4",
                },
                "installer": {
                    "type": "yum",
                    "package": "m4",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "http://savannah.gnu.org/news/?group=m4",
                "checker": {
                    "type": "apt-cache",
                    "package": "m4",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "make": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=make",
                "checker": {
                    "type": "pacman",
                    "package": "make",
                },
                "installer": {
                    "type": "pacman",
                    "package": "make",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=make",
                "checker": {
                    "type": "yum",
                    "package": "make",
                },
                "installer": {
                    "type": "yum",
                    "package": "make",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=make",
                "checker": {
                    "type": "apt-cache",
                    "package": "make",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "makeinfo": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://git.savannah.gnu.org/cgit/texinfo.git/plain/NEWS",
                "checker": {
                    "type": "pacman",
                    "package": "texinfo",
                },
                "installer": {
                    "type": "pacman",
                    "package": "texinfo",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://git.savannah.gnu.org/cgit/texinfo.git/plain/NEWS",
                "checker": {
                    "type": "yum",
                    "package": "texinfo",
                },
                "installer": {
                    "type": "yum",
                    "package": "texinfo",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://git.savannah.gnu.org/cgit/texinfo.git/plain/NEWS",
                "checker": {
                    "type": "apt-cache",
                    "package": "texinfo",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "meson": {
        "platform": "host",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://mesonbuild.com/Release-notes.html",
                "checker": {
                    "type": "yolk3k",
                    "package": "meson",
                },
                "installer": {
                    "type": "pip3",
                    "package": "meson",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://mesonbuild.com/Release-notes.html",
                "checker": {
                    "type": "pacman",
                    "package": "meson",
                },
                "installer": {
                    "type": "pacman",
                    "package": "meson",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://mesonbuild.com/Release-notes.html",
                "checker": {
                    "type": "apt-cache",
                    "package": "meson",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "ninja": {
        "platform": "host",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://ninja-build.org/",
                "prereqs": [
                    "g++",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/ninja-build/ninja.git",
                    "hint": r'v1\.\d\d\.\d+',
                },
                "installer": {
                    "type": "custom",
                    "name": "ninja",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://ninja-build.org/",
                "checker": {
                    "type": "pacman",
                    "package": "ninja",
                },
                "installer": {
                    "type": "pacman",
                    "package": "ninja",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://ninja-build.org/",
                "checker": {
                    "type": "apt-cache",
                    "package": "ninja-build",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "nuklear": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["unlicense"],
                "changelog": "https://immediate-mode-ui.github.io/Nuklear/doc/index.html#nuklear/changelog",
                "checker": {
                    "type": "git",
                    "url": "https://github.com/Immediate-Mode-UI/Nuklear.git",
                    "hint": r'4\.\d\d\.\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "nuklear",
                },
            },
        ],
    },
    "opengl": {
        "platform": "target",
        "projects": [
            {
                "arch": ["any"],
                "os": ["linux"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit", "bsd-1", "bsd-3"],
                "changelog": "https://gitlab.freedesktop.org/glvnd/libglvnd/-/tags",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-pkg-config",
                    "autoconf",
                    "libxext",
                    "xlib",
                    "xorgproto",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/glvnd/libglvnd.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "libglvnd",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["mit", "bsd-1", "bsd-3"],
                "changelog": "https://gitlab.freedesktop.org/glvnd/libglvnd/-/tags",
                "checker": {
                    "type": "pacman",
                    "package": "libglvnd",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libglvnd",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit", "sgi-b-2.0", "bsl-1.0"],
                "changelog": "https://www.mesa3d.org/news/",
                "checker": {
                    "type": "apt-cache",
                    "package": "libgl1-mesa-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
}



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.platform    import GetInstallPrefix
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             [
              "git", "clone",
              "https://github.com/Immediate-Mode-UI/Nuklear.git", "."
             ],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://github.com/Immediate-Mode-UI/Nuklear.git",
               hint=r'4\.\d\d\.\d')
             ],
             verbose)
            if not os.path.exists(installPrefix + "/include"):
                RunCommand(["mkdir", "-p", installPrefix + "/include"],
                 verbose)
                RunCommand(["cp", "./nuklear.h", installPrefix + "/include"],
                 verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(software + " not installed")

    print("VGAZER:", software, "installed")



from libvgazer.exceptions     import UnknownInstaller
from libvgazer.install.apt    import InstallApt
from libvgazer.install.cmd    import InstallCmd
from libvgazer.install.pacman import InstallPacman
from libvgazer.install.pip3   import InstallPip3
from libvgazer.install.yum    import InstallYum

class InstallersManager:
    def __init__(self):
        self.installFuncs = {
            "apt": lambda software, platformData, installerData, verbose:
                InstallApt(software, installerData["package"],
                 installerData["postInstallCommands"] if "postInstallCommands"
                  in installerData else None,
                 platformData["host"],
                 verbose),
            "cmd": lambda software, platformData, installerData, verbose:
                InstallCmd(software, installerData["cmds"], verbose),
            "not-needed": lambda software, platformData, installerData,
             verbose:
                True,
            "pacman": lambda software, platformData, installerData, verbose:
                InstallPacman(software, installerData["package"], verbose),
            "pip3": lambda software, platformData, installerData,
             verbose:
                InstallPip3(software, installerData["package"], verbose),
            "yum": lambda software, platformData, installerData, verbose:
                InstallYum(software, installerData["package"],
                 installerData["repo"] if "repo" in installerData else None,
                 installerData["postInstallCommands"] if "postInstallCommands"
                  in installerData else None,
                 verbose),
        }

    def GetInstallFunc(self, installerType):
        if installerType in self.installFuncs:
            return self.installFuncs[installerType]
        else:
            raise UnknownInstaller(
             "Unknown installer: " + installerType)



import os
import requests
from bs4 import BeautifulSoup

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.exceptions  import TarballLost
from libvgazer.store.temp  import StoreTemp
from libvgazer.working_dir import WorkingDir

def GetTarballUrl():
    response = requests.get("https://ftp.gnu.org/gnu/autoconf/")
    html = response.content.decode("utf-8")
    parsedHtml = BeautifulSoup(html, "html.parser")

    links = parsedHtml.find_all("a")

    maxMajor = -1
    maxMinor = -1
    for link in links:
        if ("autoconf" in link.text and ".tar.gz" in link.text
         and ".sig" not in link.text and "-latest" not in link.text):
            version = link.text.split("-")[1].split(".tar.gz")[0]
            splitedVersion = version.split(".")
            if int(splitedVersion[0]) > maxMajor:
                maxMajor = int(splitedVersion[0])
                maxMinor = int(splitedVersion[1])
            elif (int(splitedVersion[0]) == maxMajor
             and int(splitedVersion[1]) > maxMinor):
                maxMinor = int(splitedVersion[1])

    if (maxMajor != -1 and maxMinor != -1):
        return ("http://ftp.gnu.org/gnu/autoconf/autoconf-"
         "{major}.{minor}.tar.gz".format(major=maxMajor, minor=maxMinor))

    raise TarballLost("Tarball url not found")

def Install(software, platform, platformData, verbose):
    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    tarballUrl = GetTarballUrl()
    tarballShortFilename = tarballUrl.split("/")[-1]

    try:
        with WorkingDir(tempPath):
            RunCommand(["wget", "-P", "./", tarballUrl], verbose)
            RunCommand(
             ["tar", "--verbose", "--extract", "--gzip", "--file",
              tarballShortFilename],
             verbose)
        extractedDir = os.path.join(tempPath,
         ".".join(tarballShortFilename.split(".")[0:-2]))
        with WorkingDir(extractedDir):
            RunCommand(
             ["./configure", "--prefix=/usr/local"],
             verbose
            )
            RunCommand(
             ["make", "-j{cores_count}".format(cores_count=os.cpu_count())],
             verbose)
            RunCommand(["make", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")
